/*
** Module   : DefaultLoginPanel.java
** Abstract : implements a login panel to get the userid/pw when there is no
**            custom panel provided
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -------------------------------- Description -------------------------------
** 001 GES 20111019 First version.
** 002 OM  20131018 Updated to conform the new Authenticator API.
*/

package com.goldencode.p2j.security;

import com.goldencode.p2j.net.SessionListener;
import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.ui.chui.*;
import com.goldencode.p2j.ui.client.*;
import com.goldencode.p2j.util.*;
import java.util.*;

/**
 * Hard coded user interface which displays a prompt to obtain the login
 * credentials (userid and password). This is coded to work as a client-side
 * authentication plugin called from the security manager. While in operation,
 * the client environment and session are not fully initialized. This means
 * that processing must be limited.
 */
public class DefaultLoginPanel
implements Authenticator
{
   /** Client interface. */
   private ThinClient client = null; 

   /** Edit frame definition. */
   private ScreenDefinition editsd = null; 
   
   /** Edit frame ID. */
   private static final int EDIT_FID = 100; 
   
   /** User ID field ID. */
   private static final int USER_ID = EDIT_FID + 1;

   /** Password field ID. */
   private static final int PASSWORD_ID = USER_ID + 2; // 1 + skip

   /** Standard login status message. */
   private static final String STD_MSG = "Enter login credentials.";

   /** Invalid password status message. */
   private static final String INVALID_PASSWORD = "Invalid password!";
   
   /** Invalid user status message. */
   private static final String INVALID_USER = "Unknown user name!";

   /** The screen buffer for the editing frame. */
   private ScreenBuffer editsb = new ScreenBuffer(EDIT_FID, 3);
   
   /** The userid typed last time this was used. */
   private String lastid = null;
      
   /** The password typed last time this was used. */
   private String lastpw = null;

   /** The value of "option" node from AuthPlugin for this Authenticator */
   private String option = null;

   /**
    * Default constructor. 
    */
   public DefaultLoginPanel()
   {
      client = ThinClient.getInstance();
   }

   /**
    * Implements client side custom authentication logic.
    *
    * @param    parameters
    *           Additional configuration parameters. This plugin uses
    *           "option" parameter which is taken from the directory.
    * @param    code
    *           The result of the most recent attempt to authenticate or
    *           <code>AUTH_RESULT_NONE</code> if this is the first attempt.
    *
    * @return   Array of bytes to be transmitted to the server as the
    *           authorization input. 
    */
   @Override
   public byte[] clientAuthHook(Map<String, Object> parameters, int code)
   {
      OutputManager tk = OutputManager.instance();
      
      // push the frame definition the first time in
      if (code == AUTH_RESULT_NONE)
      {
         // create the frame definition on the fly
         editsd = new ScreenDefinition(EDIT_FID);
   
         // edit frame configuration
         FrameConfig editfc = new FrameConfig(ComponentConfig.FRAME, null);
         editfc.setRow(10);
         editfc.setBox(true);
         editfc.setCentered(true);
         editfc.setOverlay(true);
         editfc.setDcolor(Color.NORMAL);
         editfc.setSideLabels(true);
         editsd.addConfig(editfc, EDIT_FID);
         
         FillInConfig fic;
         
         // user ID entry field
         fic = new FillInConfig(ComponentConfig.FILL_IN, null);
         fic.setDataType("character");
         fic.setFormat("x(25)");
         fic.setLabel(" User ID");
         editsd.addConfig(fic, USER_ID);
         
         // set next widget position
         addSkip(editsd, 0);
   
         // password entry field
         fic = new FillInConfig(ComponentConfig.FILL_IN, null);
         fic.setDataType("character");
         fic.setFormat("x(25)");
         fic.setLabel("Password");
         fic.setBlank(true);
         editsd.addConfig(fic, PASSWORD_ID);

         // instantiate screen data
         client.pushScreenDefinition(new ScreenDefinition[] {editsd});

         // set status line for input operation
         client.statusInput(STD_MSG);
      }
      else
      {
         String msgtxt = INVALID_PASSWORD;
         
         if (code == AUTH_RESULT_INVALID_USERID)
         {
            msgtxt = INVALID_USER;
         }
         
         // display error
         client.message(msgtxt);
      }

      // process the edit frame screen buffer
      if (lastid != null)
      {
         // on errors, we have to display the previously typed userid
         editsb.putWidgetValue(USER_ID, new character(lastid));
      }

      String id = null;
      String pw = null;

      EventList el = new EventList(true);
      ScreenBuffer sbf = null;

      while (true)
      {
         // drive the logon frame I/O
         tk.setInvalidate(true);
         client.enable(EDIT_FID, null, editsb, true);
         client.clear(EDIT_FID, true);                         
         client.view(EDIT_FID, editsb, null, true);
         editsb.resetChanged();
         tk.setInvalidate(false);

         try
         {
            client.setAuthMode(true);
            sbf = client.waitFor(el, USER_ID, -1, editsb);
         }
         
         catch (ConditionException cx)
         {
            continue;
         }
         
         finally
         {
            client.setAuthMode(false);
         }

         tk.setInvalidate(true);
         client.enable(EDIT_FID, null, editsb, false);
         tk.setInvalidate(false);
   
         // inspect the input
         int keyCode = sbf.getKeyCode();

         // exit out if requested
         if (Keyboard.keyFunction(keyCode).equals("END-ERROR"))
            return new byte[] {};

         character idc = (character) sbf.getWidgetValue(USER_ID);
         
         if (idc == null && lastid == null)
            continue;

         id = (idc != null) ? idc.getValue().trim() : lastid;
         lastid = id;

         character pwc = (character) sbf.getWidgetValue(PASSWORD_ID);
         
         if (pwc == null && lastpw == null)
         {
            pw = "";
         }
         else
         {
            pw = (pwc != null) ? pwc.getValue() : lastpw;
            lastpw = pw;
         }
         
         break; 
      }

      // return a byte array
      return SecurityManager.packageIdPassword(id, pw);      
   }
   
   /**
    * Finalizes any resources allocated during authentication by the client.
    */
   @Override
   public void clientFinalize()
   {
      client.statusInputRevert();
      client.hideAll(true);
      client.destroyFrame(EDIT_FID);
   }

   /**
    * Not used for the client side.
    *
    * @param    auth
    *           The authorization input from the client.
    * @param    entity
    *           Additional configuration parameters taken from the directory.
    *
    * @return   Always <code>null</code>.
    */
   @Override
   public AuthenticationResponse serverAuthHook(byte[] auth, String entity)
   {
      return null;
   }

   /**
    * Always returns <code>null</code>.
    *
    * @return  Always <code>null</code>.
    */
   @Override
   public SessionListener getSessionListener()
   {
      return null;
   }

   /**
    * Returns a set of entities that this class handles.
    *
    * @return  A set with the value of option that was configured or an empty set if option was
    *          not configured.
    */
   @Override
   public Set<String> getAuthenticationEntities()
   {
      Set<String> ret = new HashSet<>();
      if (option != null)
      {
         ret.add(option);
      }
      return ret;
   }

   /**
    * Configures the Authenticator by setting the "option" parameter from directory.xml.
    *
    * @param   option
    *          The value of "option" entry for the auth plugin.
    */
   @Override
   public void configure(String option)
   {
      this.option = option;
   }

   /**
    * Add specified number of empty lines
    *
    * @param   sd
    *          The frame to which to add a skip.
    * @param   lines
    *          Lines to skip.
    */
   private void addSkip(ScreenDefinition sd, int lines)
   {
      SkipConfig sc = new SkipConfig(ComponentConfig.SKIP, null);
      sc.setVertical(true);   // skip not space
      sc.setHeight(lines);
      sd.addConfig(sc);
   }
}
